package lrs.visitor;

import lrs.*;

import junit.framework.TestCase;

public class Test_All extends TestCase {
  

  public void test_LRSEqual() {
    IAlgo algo = LRSEqual.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    LRStruct lrs2 = new LRStruct();
    
    assertEquals("() vs. ()", true, lrs1.execute(algo, lrs2));
    
    lrs1.insertFront("a");
    assertEquals("(a) vs. ()", false, lrs1.execute(algo, lrs2));
    assertEquals("() vs. (a)", false, lrs2.execute(algo, lrs1));
    
    
    lrs2.insertFront("b");
    assertEquals("(a) vs. (b)", false, lrs1.execute(algo, lrs2));

    lrs2.removeFront();
    lrs2.insertFront("a");
    assertEquals("(a) vs. (a)", true, lrs1.execute(algo, lrs2));

    lrs1.insertFront("c");
    assertEquals("(c a) vs. (a)", false, lrs1.execute(algo, lrs2));
    
    lrs2.insertFront("d");
    assertEquals("(c a) vs. (d a)", false, lrs1.execute(algo, lrs2));
    
    lrs2.removeFront();
    lrs2.insertFront("c");
    assertEquals("(c a) vs. (c a)", true, lrs1.execute(algo, lrs2)); 
  }

  public void test_LRSReverse() {
    IAlgo algo = LRSReverse.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    
    assertEquals("()", "()", lrs1.execute(algo).toString());
    
    lrs1.insertFront("a");
    assertEquals("(a)", "(a)", lrs1.execute(algo).toString());
    
    
    lrs1.insertFront("b");
    assertEquals("(b a)", "(a b)", lrs1.execute(algo).toString());

    lrs1.insertFront("c");
    assertEquals("(c b a)", "(a b c)", lrs1.execute(algo).toString());   
  }  
  
  public void test_LRSPalindrome() {
    IAlgo algo = LRSPalindrome.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    
    assertEquals("()", true, lrs1.execute(algo));
    
    lrs1.insertFront("a");
    assertEquals("(a)", true, lrs1.execute(algo));
    
    
    lrs1.insertFront("b");
    assertEquals("(b a)", false, lrs1.execute(algo));

    lrs1.insertFront("a");
    assertEquals("(a b a)", true , lrs1.execute(algo)); 
    
    lrs1.removeFront();
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    assertEquals("(a b b a)", true , lrs1.execute(algo));  
  }    
  
  public void test_LRSPalin() {
    IAlgo algo = LRSPalin.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    
    assertEquals("()", true, lrs1.execute(algo));
    
    lrs1 = new LRStruct();
    lrs1.insertFront("a");
    assertEquals("(a)", true, lrs1.execute(algo));
    
    lrs1 = new LRStruct();
    lrs1.insertFront("a");    
    lrs1.insertFront("b");
    assertEquals("(b a)", false, lrs1.execute(algo));

    lrs1 = new LRStruct();
    lrs1.insertFront("a");    
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    assertEquals("(a b a)", true , lrs1.execute(algo)); 
    
    lrs1 = new LRStruct();
    lrs1.insertFront("a");    
    lrs1.insertFront("b");
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    assertEquals("(a b b a)", true , lrs1.execute(algo));  
  }  
}
