package lrs.visitor;
import lrs.*;
import java.util.*;

/**
 * Swaps the first element of an LRStruct list, if 
 * it exists, with a randomly chosen element in the
 * rest of the list.  Mutates the host list.  Returns
 * the host list.
 */
public class RandomSwap implements IAlgo {
  
  /**
   * Random number generator.  
   * rand.nextInt(anInt) will return an int between zero, inclusive, and 
   * anInt, exclusive.
   */
  Random rand;
  
  /**
   * This constructor initializes the Random number generator to a new, but 
   * undetermined generator.
   */
  public RandomSwap() {
    this(new Random()); //delegates to the other constructor
  }
  
  /**
   * This constructor allows the Random number generator to be initialized to 
   * a specific generator that generates a known sequence.  
   * This is used to enable consistent testing.
   */
  public RandomSwap(Random rand) {
    this.rand = rand;
  }
  
  /**
   * Does nothing
   * @param host an empty LRStruct.
   * @param n not used
   * @return LRStruct host
   */
  public Object emptyCase(LRStruct host, Object... n) {
    // STUDENT TO COMPLETE
    return host;
  }
  
  /**
   * Randomly swaps the first element with a random element in the rest of the list.
   * @param host a non-empty LRStruct.
   * @param n not used
   * @return LRStruct host
   */
  public Object nonEmptyCase(final LRStruct host, Object... n) {
    // STUDENT TO COMPLETE    
    host.getRest().execute(new IAlgo() {
      public Object emptyCase(LRStruct h, Object... n) {
        return rand.nextInt((Integer)n[0]);
      }
      
      public Object nonEmptyCase(LRStruct h, Object... n) {
        int i = (Integer) h.getRest().execute(this, 1+(Integer)n[0]);
        if(0==i) {
          Object f0 = host.getFirst();
          host.setFirst(h.getFirst());
          h.setFirst(f0);
        }
        
        return i-1;
      }
    }, 1);
    return host;
  }
}